/**
 * Created by raos on 6/28/17.
 */

/* @depend QfqEvents.js */

/**
 * Qfq Namespace
 *
 * @namespace QfqNS
 */
var QfqNS = QfqNS || {};

(function (n) {
    'use strict';

    n.Dirty = function (dirtyUrl) {
        this.dirtyUrl = dirtyUrl;

        this.eventEmitter = new EventEmitter();
    };

    n.Dirty.events = {
        STARTED: 'dirty.notify.started',
        SUCCESS: 'dirty.notify.success',
        ENDED: 'dirty.notify.ended',
        FAILED: 'dirty.notify.failed',
        DENIED: 'dirty.notify.denied'
    };

    n.Dirty.prototype.on = n.EventEmitter.onMixin;

    /**
     * Notify the server that SIP is becoming dirty.
     *
     * @param sip {string} sip.
     * @public
     */
    n.Dirty.prototype.notifyServer = function (sip) {
        var eventData;

        if (!this.dirtyUrl) {
            n.Log.debug("notifyServer: cannot notify server, no dirtyUrl.");
            return;
        }
        eventData = n.EventEmitter.makePayload(this, null);
        this.eventEmitter.emitEvent(n.Dirty.events.STARTED, eventData);
        $.ajax({
            url: this.makeUrl(sip),
            type: 'GET',
            cache: false
        })
            .done(this.ajaxSuccessHandlerNotify.bind(this))
            .fail(this.ajaxErrorHandlerNotify.bind(this));
    };

    /**
     * @private
     * @param data
     * @param textStatus
     * @param jqXHR
     */
    n.Dirty.prototype.ajaxSuccessHandlerNotify = function (data, textStatus, jqXHR) {
        var eventData = n.EventEmitter.makePayload(this, data);
        if (data.status && data.status === "success") {
            this.eventEmitter.emitEvent(n.Dirty.events.SUCCESS, eventData);
        } else {
            this.eventEmitter.emitEvent(n.Dirty.events.DENIED, eventData);
        }

        this.eventEmitter.emitEvent(n.Dirty.events.ENDED, eventData);
    };

    /**
     * @private
     * @param jqXHR
     * @param textStatus
     * @param errorThrown
     */
    n.Dirty.prototype.ajaxErrorHandlerNotify = function (jqXHR, textStatus, errorThrown) {
        var eventData = n.EventEmitter.makePayload(this, null, {
            textStatus: textStatus,
            errorThrown: errorThrown,
            jqXHR: jqXHR
        });
        this.eventEmitter.emitEvent(n.Dirty.events.FAILED, eventData);
        this.eventEmitter.emitEvent(n.Dirty.events.ENDED, eventData);
    };

    n.Dirty.prototype.makeUrl = function (sip) {
        return this.dirtyUrl + "?s=" + encodeURIComponent(sip);
    };
})(QfqNS);
