/**
 * Created by raos on 6/28/17.
 */

describe("Dirty", function () {
    "use strict";

    var dirtyUnderTest;
    var dirtyUnderTestNoUrl;
    var dirtyEventSpy;

    beforeEach(function () {
        jasmine.Ajax.install();
        dirtyUnderTest = new QfqNS.Dirty("/ajax/mocked");
        dirtyUnderTestNoUrl = new QfqNS.Dirty(null);
        dirtyEventSpy = {
            dirtyNotifySuccess: function () {
            },
            dirtyNotifyDenied: function () {
            },
            dirtyNotifyEnded: function () {
            },
            dirtyNotifyStarted: function () {
            },
            dirtyNotifyFailed: function () {
            }
        };
        spyOn(dirtyEventSpy, 'dirtyNotifySuccess');
        spyOn(dirtyEventSpy, 'dirtyNotifyDenied');
        spyOn(dirtyEventSpy, 'dirtyNotifyEnded');
        spyOn(dirtyEventSpy, 'dirtyNotifyStarted');
        spyOn(dirtyEventSpy, 'dirtyNotifyFailed');

        dirtyUnderTest.on(QfqNS.Dirty.events.STARTED, dirtyEventSpy.dirtyNotifyStarted);
        dirtyUnderTest.on(QfqNS.Dirty.events.ENDED, dirtyEventSpy.dirtyNotifyEnded);
        dirtyUnderTest.on(QfqNS.Dirty.events.FAILED, dirtyEventSpy.dirtyNotifyFailed);
        dirtyUnderTest.on(QfqNS.Dirty.events.SUCCESS, dirtyEventSpy.dirtyNotifySuccess);
        dirtyUnderTest.on(QfqNS.Dirty.events.DENIED, dirtyEventSpy.dirtyNotifyDenied);

        dirtyUnderTestNoUrl.on(QfqNS.Dirty.events.STARTED, dirtyEventSpy.dirtyNotifyStarted);
        dirtyUnderTestNoUrl.on(QfqNS.Dirty.events.ENDED, dirtyEventSpy.dirtyNotifyEnded);
        dirtyUnderTestNoUrl.on(QfqNS.Dirty.events.FAILED, dirtyEventSpy.dirtyNotifyFailed);
        dirtyUnderTestNoUrl.on(QfqNS.Dirty.events.SUCCESS, dirtyEventSpy.dirtyNotifySuccess);
        dirtyUnderTestNoUrl.on(QfqNS.Dirty.events.DENIED, dirtyEventSpy.dirtyNotifyDenied);
    });

    afterEach(function () {
        jasmine.Ajax.uninstall();
    });

    it("should call the proper url", function () {
        dirtyUnderTest.notifyServer("mysip");


        // We use `startsWith()` because `$.ajax` appends a timestamp to the URL to avoid caching.
        expect(jasmine.Ajax.requests.mostRecent().url).toMatch(/^\/ajax\/mocked\?s=mysip/);
    });

    it("should call handlers upon successful notification", function () {

        dirtyUnderTest.notifyServer("mysip");
        jasmine.Ajax.requests.mostRecent().respondWith({
            "status": 200,
            "contentType": 'application/json',
            "responseText": '{ "status": "success", "message": "ok" }'
        });

        expect(dirtyEventSpy.dirtyNotifyStarted).toHaveBeenCalledTimes(1);
        expect(dirtyEventSpy.dirtyNotifySuccess).toHaveBeenCalledTimes(1);
        expect(dirtyEventSpy.dirtyNotifySuccess.calls.argsFor(0)[0].data).toEqual({
            "status": "success",
            "message": "ok"
        });
        expect(dirtyEventSpy.dirtyNotifyEnded).toHaveBeenCalledTimes(1);

        expect(dirtyEventSpy.dirtyNotifyDenied).not.toHaveBeenCalled();
        expect(dirtyEventSpy.dirtyNotifyFailed).not.toHaveBeenCalled();
    });

    it("should call handlers upon denied notification", function () {

        dirtyUnderTest.notifyServer("mysip");
        jasmine.Ajax.requests.mostRecent().respondWith({
            "status": 200,
            "contentType": 'application/json',
            "responseText": '{ "status": "conflict", "message": "already open" }'
        });

        expect(dirtyEventSpy.dirtyNotifyStarted).toHaveBeenCalledTimes(1);
        expect(dirtyEventSpy.dirtyNotifyDenied).toHaveBeenCalledTimes(1);
        expect(dirtyEventSpy.dirtyNotifyDenied.calls.argsFor(0)[0].data).toEqual({
            "status": "conflict",
            "message": "already open"
        });
        expect(dirtyEventSpy.dirtyNotifyEnded).toHaveBeenCalledTimes(1);

        expect(dirtyEventSpy.dirtyNotifySuccess).not.toHaveBeenCalled();
        expect(dirtyEventSpy.dirtyNotifyFailed).not.toHaveBeenCalled();
    });

    it("should call handlers upon error", function () {
        dirtyUnderTest.notifyServer("mysip");
        jasmine.Ajax.requests.mostRecent().respondWith({
            "status": 400,
            "contentType": 'application/json',
            "responseText": '{ "status": "error", "message": "foobar" }'
        });

        expect(dirtyEventSpy.dirtyNotifyStarted).toHaveBeenCalledTimes(1);
        expect(dirtyEventSpy.dirtyNotifyFailed).toHaveBeenCalledTimes(1);
        expect(dirtyEventSpy.dirtyNotifyEnded).toHaveBeenCalledTimes(1);

        expect(dirtyEventSpy.dirtyNotifySuccess).not.toHaveBeenCalled();
        expect(dirtyEventSpy.dirtyNotifyDenied).not.toHaveBeenCalled();
    });

    it("should not make ajax request on empty/null url", function () {
        dirtyUnderTestNoUrl.notifyServer("mysip");

        expect(jasmine.Ajax.requests.mostRecent()).toBeUndefined();

        expect(dirtyEventSpy.dirtyNotifyStarted).not.toHaveBeenCalled();
        expect(dirtyEventSpy.dirtyNotifyFailed).not.toHaveBeenCalled();
        expect(dirtyEventSpy.dirtyNotifyEnded).not.toHaveBeenCalled();
        expect(dirtyEventSpy.dirtyNotifySuccess).not.toHaveBeenCalled();
        expect(dirtyEventSpy.dirtyNotifyDenied).not.toHaveBeenCalled();
    });

});
